// -*- coding:unix; mode:c++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
#ifndef VIETNAMESE_CHARSET_DATA_H
#define VIETNAMESE_CHARSET_DATA_H

// This header defines some special characters
const StdVnChar StdStartQuote = (VnStdCharOffset + 201);  // 0x93 in the Western charset
// 201 is the offset of character 0x93 (start quote) in Vn charsets
const StdVnChar StdEndQuote = (VnStdCharOffset + 202); // 0x94 in the Western charset
const StdVnChar StdEllipsis = (VnStdCharOffset + 190); // 0x85 in Western charet.

#endif
